/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s): 
 */

package org.objectweb.dream.cache.replacement.lib;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.cache.replacement.api.ReplaceableCacheEntry;

/**
 * Replacement Manager with the "first-in first-out" policy
 * 
 * @author E. Bruneton
 * @author P. Garcia
 */
@DreamComponent
public class FIFOReplacementManager extends AbstractReplacementManager {

	/**
	 * Logical clock used to assign an access time to entries
	 */
	protected long date;

	public FIFOReplacementManager() {
		date = Long.MIN_VALUE;
	}

	protected void add(ReplaceableCacheEntry entry) {
		entry.setCeAge(date++);
	}

	protected void touch(ReplaceableCacheEntry entry) {
		// does nothing
	}
}
