/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.content;

import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.Controller;
import org.objectweb.fractal.julia.InitializationContext;

/**
 * Defines a <code>weaveableOptCC</code>.
 */
public abstract class UseOptContentControllerMixin implements Controller
{
  /**
   * Private constructor
   *
   */
  private UseOptContentControllerMixin()
  {
  }

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * The optional {@link ContentController}interface of the component to which
   * this controller object belongs.
   */

  public ContentController weaveableOptCC;

  /**
   * Initializes the fields of this mixin and then calls the overriden method.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */

  public void initFcController(final InitializationContext ic)
      throws InstantiationException
  {
    try
    {
      weaveableOptCC = (ContentController) ic
          .getInterface("content-controller");
    }
    catch (InstantiationException e)
    {
      weaveableOptCC = null;
    }
    _super_initFcController(ic);
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The {@link Controller#initFcController initFcController}method overriden
   * by this mixin.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */

  public abstract void _super_initFcController(InitializationContext ic)
      throws InstantiationException;
}