/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.factory;

import java.net.InetAddress;
import java.net.UnknownHostException;

import org.objectweb.fractal.api.control.NameController;

/**
 * Override {@link NameController#getFcName()} method to returns a default name,
 * if it is not set elsewhere. The default name is the local canonical host
 * name.
 * 
 * @see InetAddress#getLocalHost()
 * @see InetAddress#getCanonicalHostName()
 */
public abstract class NameFactoryMixin implements NameController
{
  /**
   * Private Constructor
   *
   */
  private NameFactoryMixin()
  {
  }

  // ---------------------------------------------------------------------------
  // Fields and methods added and overridden by the mixin class
  // ---------------------------------------------------------------------------

  /**
   * @see NameController#getFcName()
   */
  public String getFcName()
  {
    String name = _super_getFcName();
    if (name == null)
    {
      try
      {
        return InetAddress.getLocalHost().getCanonicalHostName();
      }
      catch (UnknownHostException e)
      {
        // ignored
      }
    }
    return name;
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The {@link NameController#getFcName getFcName}method overridden by this
   * mixin.
   * 
   * @see NameController#getFcName
   */
  public abstract String _super_getFcName();

}
