/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.logger;

import org.objectweb.fractal.julia.InitializationContext;

/**
 * Basic mixin implementing the <code>LoggerInternalBaseName</code> interface.
 * <br>
 * <br>
 * <b>Requirements </b>
 * <ul>
 * <li>this mixin requires the {@link BasicLoggerControllerMixin }mixin.</li>
 * </ul>
 */
public abstract class BasicLoggerInternalBaseNameMixin
    implements
      LoggerInternalBaseName
{

  boolean externalAssignedBaseName;

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public void initFcController(InitializationContext ic)
      throws InstantiationException
  {
    _super_initFcController(ic);
    externalAssignedBaseName = false;
  }

  /**
   * @see LoggerController#setBaseName(String)
   */
  public void setBaseName(String name)
  {
    externalAssignedBaseName = true;
    _super_setBaseName(name);
  }

  /**
   * @see LoggerInternalBaseName#internalSetBaseName(String)
   */
  public void internalSetBaseName(String name)
  {
    if (!externalAssignedBaseName)
    {
      _super_setBaseName(name);
    }
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The
   * {@link org.objectweb.fractal.julia.Controller#initFcController(InitializationContext) initFcController }
   * method overriden by this mixin.
   * 
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public abstract void _super_initFcController(InitializationContext ic)
      throws InstantiationException;

  /**
   * The {@link LoggerController#setBaseName(String) setBaseName }method
   * overriden by this mixin
   * 
   * @see LoggerController#setBaseName(String)
   */
  public abstract void _super_setBaseName(String name);

}