/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import java.util.HashMap;
import java.util.Map;

class ChunkFactoryRepository
{
  private Map<Class< ? >, ChunkFactoryReference< ? >> factories = new HashMap<Class< ? >, ChunkFactoryReference< ? >>();

  /**
   * Add a factory to the repository
   * @param <T> type of the chunk extending
   * @param chunkType class correponding to the type of the chunk
   * @param chunkFactory factory to add
   */
  <T extends AbstractChunk<T>> void addFactory(final Class<T> chunkType,
      final ChunkFactoryReference<T> chunkFactory)
  {
    factories.put(chunkType, chunkFactory);
  }

  /**
   * 
   * @param <T>
   * @param chunkType
   * @return
   */
  @SuppressWarnings("unchecked")
  <T extends AbstractChunk<T>> ChunkFactoryReference<T> getFactory(
      final Class<T> chunkType)
  {
    return (ChunkFactoryReference<T>) factories.get(chunkType);
  }
}
