/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.pool.ObjectPoolItf;
import org.objectweb.dream.pool.ObjectPoolSoftReference;
import org.objectweb.dream.pool.Recyclable;
import org.objectweb.dream.pool.RecyclableFactoryType;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * Extension of the basic message manager. This implementation recycle messages
 * and chunks in object pools.
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class MessageManagerPoolingImpl extends MessageManager
    implements
      RecyclableFactoryType
{
  @Requires(name = ObjectPoolItf.ITF_NAME)
  ObjectPoolItf messagePoolItf;

  // ---------------------------------------------------------------------------
  // Overridden MessageManagerType methods
  // ---------------------------------------------------------------------------

  /**
   * @see MessageManagerType#createMessage()
   */
  @Override
  public Message createMessage()
  {
    Message message = (Message) messagePoolItf.newInstance();
    message.incrementReferenceCounter();
    return message;
  }

  /**
   * @see MessageManager#recycleMessageInstance(Message)
   */
  @Override
  protected void recycleMessageInstance(final Message message)
  {
    messagePoolItf.recycleInstance(message);
  }

  /**
   * @see MessageManager#createChunkFactory(Class)
   */
  @Override
  protected <T extends AbstractChunk<T>> ChunkFactoryReference<T> createChunkFactory(
      final Class<T> chunkType)
  {
    ObjectPoolSoftReference pool = new ObjectPoolSoftReference();
    ChunkFactoryReference<T> factory;
    try
    {
      factory = new BasicChunkFactoryPoolingImpl<T>(chunkType, this, pool);
    }
    catch (Exception e)
    {
      throw new IllegalArgumentException(e);
    }
    try
    {
      ((BindingController) pool)
          .bindFc(RecyclableFactoryType.ITF_NAME, factory);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
    return factory;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the RecyclableFactoryType interface
  // ---------------------------------------------------------------------------

  /**
   * @see RecyclableFactoryType#newInstance()
   */
  public Recyclable newInstance()
  {
    return new BasicMessageImpl(this);
  }

}
