/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Iterator;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.AbstractChunk;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * Optimized implementation of message codec for extensible message using object
 * stream. <br>
 */

@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class MessageCodecPrimitiveObjectStream
    extends
      AbstractDelegatingMessageCodec

{
  @Requires(name = MessageManagerType.ITF_NAME)
  protected MessageManagerType messageManagerItf;
  
  // ---------------------------------------------------------------------------
  // Implementation of the MessageCodec interface.
  // ---------------------------------------------------------------------------

  /**
   * @see MessageCodec#decode(CodecInputOutput)
   */
  @SuppressWarnings("unchecked")
  public Message decode(CodecInputOutput cio) throws IOException
  {
    Message message = delegateMessageCodecItf.decode(cio);
    ObjectInput objectInput = (ObjectInput) cio.getInput();

    boolean hasMoreChunk = objectInput.readBoolean();
    try
    {
      while (hasMoreChunk)
      {
        String chunkName = objectInput.readUTF();
        String chunkClassName = objectInput.readUTF();
        Class chunkClass = Class.forName(chunkClassName);
        ChunkFactoryReference f = messageManagerItf
            .getChunkFactory(chunkClass);
        AbstractChunk chunk = messageManagerItf.createChunk(f);
        chunk.readExternal(objectInput);
        messageManagerItf.addChunk(message, chunkName, chunk);
        hasMoreChunk = objectInput.readBoolean();
      }
    }
    catch (ClassNotFoundException e)
    {
      throw new IOException("Unable to decode message : " + e.getMessage());
    }
    return message;
  }

  /**
   * @see MessageCodec#encode(CodecInputOutput, Message)
   */
  public void encode(CodecInputOutput cio, Message message)
      throws IOException
  {
    delegateMessageCodecItf.encode(cio, message);
    ObjectOutput objectOutput = (ObjectOutput) cio.getOutput();
    Iterator<String> iterator = messageManagerItf.getChunkNameIterator(message);
    while (iterator.hasNext())
    {
      String chunkName = iterator.next();
      AbstractChunk chunk = messageManagerItf.getChunk(message, chunkName);
      objectOutput.writeBoolean(true);
      objectOutput.writeUTF(chunkName);
      objectOutput.writeUTF(chunk.getClass().getName());
      ((Externalizable) chunk).writeExternal(objectOutput);
    }
    objectOutput.writeBoolean(false);
  }
}
