/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec.compression;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.dreamannotation.DreamMonolog;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.codec.AbstractDelegatingMessageCodec;
import org.objectweb.dream.message.codec.CodecInputOutput;
import org.objectweb.dream.util.LoggerUtil;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Message Codec which manage zip entries
 * 
 * @author pgarcia
 */
@DreamComponent
public class MessageCodecHelperZipStream extends AbstractDelegatingMessageCodec
{
  private final static int  END_OFF_ZIP_ENTRY = -1;

  @Service
  Component                 reference;

  @DreamMonolog
  protected Logger          logger;

  protected int             zipEntryName      = 0;

  protected ZipOutputStream zipOutputStream;

  protected ZipInputStream  zipInputStream;

  /**
   * Get next zip entry in the inputStream <br />
   * 
   * @return null
   */
  public Message decode(CodecInputOutput cio) throws IOException
  {
    // LoggerUtil.start(logger);
    // ZipInputStream zipInputStream = null;
    // if (cio instanceof ZipCodecInputOutput)
    // {
    // zipInputStream = ((ZipCodecInputOutput) cio).getZipInputStream();
    // ZipEntry z = zipInputStream.getNextEntry();
    // if (logger.isLoggable(BasicLevel.DEBUG)){
    // logger.log(BasicLevel.DEBUG, "Decoded zip entry : " + z.getName());
    // }
    // }
    // else
    // {
    // throw new IOException("Unknown codec : " +
    // cio.getClass().getSimpleName());
    // }
    // LoggerUtil.returnAfter(logger);
    // Message m = delegateMessageCodecItf.decode(cio);
    //    
    // // It must be the end for the zipEntry
    // if (zipInputStream.read() != END_OFF_ZIP_ENTRY)
    // throw new IOException("It must the end of the zip input");
    // return m;

    Object input = cio.getInput();
    if (input instanceof InputStream)
    {
      zipInputStream = new ZipInputStream((InputStream) input);
      cio.setInput(zipInputStream);
    }
    else if (!(input instanceof ObjectInputStream))
    {
      logger.log(BasicLevel.WARN, "The component " + this.getClass().getName()
          + " is useless with the " + cio.getClass().getSimpleName()
          + " codec input/ouput ");
    }
    ZipEntry z = zipInputStream.getNextEntry();
    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      logger.log(BasicLevel.DEBUG, "Decoded zip entry : " + z.getName());
    }
    Message m = delegateMessageCodecItf.decode(cio);

    // It must be the end for the zipEntry
    if (zipInputStream.read() != END_OFF_ZIP_ENTRY)
      throw new IOException("It must the end of the zip input");
    return m;
  }

  /**
   * Put next zip entry in the outputStream
   */
  public void encode(CodecInputOutput cio, Message message) throws IOException
  {
    // LoggerUtil.start(logger);
    // ZipOutputStream zipOutputStream = null;
    // if (cio instanceof ZipCodecInputOutput)
    // {
    // zipOutputStream = ((ZipCodecInputOutput) cio).getZipOutputStream();
    // if (logger.isLoggable(BasicLevel.DEBUG)){
    // logger.log(BasicLevel.DEBUG, "New zip entry : " + zipEntryName);
    // }
    // zipOutputStream.putNextEntry(new ZipEntry(zipEntryName++ + "" ));
    // }
    // else
    // {
    // throw new IOException("Unknown codec : " +
    // cio.getClass().getSimpleName());
    // }
    // delegateMessageCodecItf.encode(cio, message);
    // zipOutputStream.closeEntry();
    // LoggerUtil.end(logger);

    Object output = cio.getOutput();
    if (output instanceof OutputStream)
    {
      // creation of the zip output stream
      zipOutputStream = new ZipOutputStream((OutputStream) output);
      cio.setOutput(zipOutputStream);
    }
    else if (!(output instanceof ObjectOutputStream))
    {
      logger.log(BasicLevel.WARN, "The component " + this.getClass().getName()
          + " is useless with the " + cio.getClass().getSimpleName()
          + " codec input/ouput ");
      delegateMessageCodecItf.encode(cio, message);
      return;
    }
    zipOutputStream.putNextEntry(new ZipEntry(zipEntryName++ + ""));
    delegateMessageCodecItf.encode(cio, message);
    zipOutputStream.closeEntry();
  }

}
