/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s):
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec.string;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.dreamannotation.DreamMonolog;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.codec.AbstractDelegatingMessageCodec;
import org.objectweb.dream.message.codec.CodecInputOutput;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.Attribute;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * @author pgarcia
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class MessageCodecHelperReaderWriterStream
    extends
      AbstractDelegatingMessageCodec
{
  @Service
  Component ref;
  
  @DreamMonolog
  Logger logger;

  // ---------------------------------------------------------------------------
  // Attribute field
  // ---------------------------------------------------------------------------
  @Attribute(argument = "reuseObjectStream")
  boolean         reuseObjectStream;
  
  /**
   * @see org.objectweb.dream.message.codec.MessageCodec#decode(org.objectweb.dream.message.codec.CodecInputOutput)
   */
  public Message decode(CodecInputOutput cio) throws IOException
  {
    Object input = cio.getInput();
    InputStreamReader isr;
    if (!(input instanceof ObjectInputStream) && input instanceof InputStream)
    {
      isr = new InputStreamReader((InputStream) input);
      cio.setInput(isr);
    }
    else if (!(input instanceof ObjectInputStream))
    {
      logger.log(BasicLevel.WARN, "The component " + this.getClass().getName()
          + " is useless with the " + cio.getClass().getSimpleName()
          + " codec input/ouput ");
    }
    Message message = delegateMessageCodecItf.decode(cio);
    if (!reuseObjectStream)
    {
      cio.setInput(input);
    }
    return message;
  }

  /**
   * @see org.objectweb.dream.message.codec.MessageCodec#encode(org.objectweb.dream.message.codec.CodecInputOutput,
   *      org.objectweb.dream.message.Message)
   */
  public void encode(CodecInputOutput cio, Message message) throws IOException
  {
    Object output = cio.getOutput();
    OutputStreamWriter osw;
    if (!(output instanceof ObjectOutputStream) && output instanceof OutputStream)
    {
      osw = new OutputStreamWriter((OutputStream) output);
      cio.setOutput(osw);
    }
    else if (!(output instanceof ObjectOutputStream))
    {
      logger.log(BasicLevel.WARN, "The component " + this.getClass().getName()
          + " is useless with the " + cio.getClass().getSimpleName()
          + " codec input/ouput ");
    }
    delegateMessageCodecItf.encode(cio, message);
    if (!reuseObjectStream)
    {
      cio.setOutput(output);
    }
  }

}
