/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec.string;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Reader;
import java.io.Writer;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.codec.AbstractDelegatingMessageCodec;
import org.objectweb.dream.message.codec.CodecInputOutput;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.Attribute;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;

import com.thoughtworks.xstream.XStream;

/**
 * @author pgarcia
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class MessageCodecHelperXStream extends AbstractDelegatingMessageCodec
{
  // ---------------------------------------------------------------------------
  // Attribute field
  // ---------------------------------------------------------------------------
  @Attribute(argument = "reuseObjectStream")
  boolean         reuseObjectStream;

  /**
   * XStream tool to serialize/deserialize object to/from xml.
   * ObjectOutputStream and ObjectInputStream redefine in this library are used
   * here
   */
  private XStream xstream;

  /**
   * @see org.objectweb.dream.message.codec.MessageCodec#decode(org.objectweb.dream.message.codec.CodecInputOutput)
   */
  public Message decode(CodecInputOutput cio) throws IOException
  {
    Object input = cio.getInput();
    ObjectInputStream ois;
    if (input instanceof Reader)
    {
      ois = xstream.createObjectInputStream((Reader) input);
      cio.setInput(ois);
    }
    else if (!(input instanceof ObjectInputStream))
    {
      throw new IOException("Unknown input : " + input);
    }
    Message message = delegateMessageCodecItf.decode(cio);
    if (!reuseObjectStream)
    {
      cio.setInput(input);
    }
    return message;
  }

  /**
   * Create the xstream {@link ObjectOutputStream} if the output field of the
   * {@link CodecInputOutput} is an instance of {@link Writer} This
   * {@link ObjectOutputStream} is then assigned to the codec.
   */
  public void encode(CodecInputOutput cio, Message message) throws IOException
  {
    Object output = cio.getOutput();
    ObjectOutputStream oos;
    if (output instanceof Writer)
    {
      oos = xstream.createObjectOutputStream((Writer) output);
      cio.setOutput(oos);
    }
    else if (output instanceof ObjectOutputStream)
    {
      oos = (ObjectOutputStream) output;
    } else {
      throw new IOException("Unknown output : " + output);
    }
    delegateMessageCodecItf.encode(cio, message);
    oos.flush();
    if (!reuseObjectStream)
    {
      cio.setOutput(output);
    }
  }

  // ---------------------------------------------------------------------------
  // Lifecycle methods
  // ---------------------------------------------------------------------------

  /**
   * Export the channel
   */
  @LifeCycle(on = LifeCycleType.START)
  public void startFcHandler() throws IllegalLifeCycleException
  {
    xstream = new XStream();
  }
}
