/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.util;

import org.objectweb.dream.control.activity.task.TaskController;
import org.objectweb.dream.control.logger.LoggerController;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.julia.Julia;

/**
 * Provides static methods to access control interfaces of Dream components. It
 * also provides an enumeration of Dream Controllers for fraclet annotations.
 * This class is provided in addition to the
 * {@link org.objectweb.fractal.util.Fractal }class.
 */
public final class Dream
{

  /**
   * Private constructor
   */
  private Dream()
  {
  }

  /**
   * Controller Name of Primitive Components
   */
  public static final String PRIMITIVE_CONTROLLER                    = "dreamPrimitive";
  
  /**
   * Controller Name of Active Primitive Components
   */
  public static final String ACTIVE_PRIMITIVE_CONTROLLER             = "activeDreamPrimitive";
  
  /**
   * Controller Name of Unstoppable Primitive Components
   */
  public static final String UNSTOPPABLE_PRIMITIVE_CONTROLLER        = "dreamUnstoppablePrimitive";
  
  /**
   * Controller Name of Active Unstoppable Primitive Components
   */
  public static final String ACTIVE_UNSTOPPABLE_PRIMITIVE_CONTROLLER = "activeDreamUnstoppablePrimitive";
  
  /**
   * Default value for "julia.config" system property
   */
  public static final String JULIA_CONFIG = "julia-dream-bundled.cfg,julia-dream-activities-bundled.cfg";
  
  /**
   * Default Fractal Provider : Julia
   */
  public static final String DEFAULT_FRACTAL_PROVIDER = Julia.class
                                                    .getCanonicalName();

  /**
   * Returns the logger controller interface of the given component.
   * 
   * @param component a component interface
   * @return the logger controller interface of the given component.
   * @throws NoSuchInterfaceException if the givn component doesn't have a
   *           logger controller.
   */
  public static LoggerController getLoggerController(Component component)
      throws NoSuchInterfaceException
  {
    return (LoggerController) component.getFcInterface("logger-controller");
  }

  /**
   * Returns the task controller interface of the given component.
   * 
   * @param component a component interface
   * @return the task controller interface of the given component.
   * @throws NoSuchInterfaceException if the givn component doesn't have a task
   *           controller.
   */
  public static TaskController getTaskController(Component component)
      throws NoSuchInterfaceException
  {
    return (TaskController) component.getFcInterface("task-controller");
  }
}