/**
 * Dream
 * Copyright (C) 2003-2007 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s):
 */

package org.objectweb.dream.util;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class defines utility methods for logging.
 */
public final class LoggerUtil
{
  /**
   * Private constructor
   */
  private LoggerUtil()
  {
  }

  /**
   * To log the start of a method
   * 
   * @param logger
   */
  public final static void start(Logger logger)
  {
    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      StackTraceElement ste = Thread.currentThread().getStackTrace()[3];
      String className = ste.getClassName();
      logger.log(BasicLevel.DEBUG, className.substring(className.lastIndexOf(".")+1) + "." + ste.getMethodName() + " START");
    }
  }

  /**
   * To log the stop of a method
   * 
   * @param logger
   */
  public final static void end(Logger logger)
  {
    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      StackTraceElement ste = Thread.currentThread().getStackTrace()[3];
      String className = ste.getClassName();
      logger.log(BasicLevel.DEBUG, className.substring(className.lastIndexOf(".")+1) + "." + ste.getMethodName() + " END");
    }
  }

  /**
   * To log the return of a method
   * 
   * @param logger
   */
  public final static void returnAfter(Logger logger)
  {
    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      StackTraceElement ste = Thread.currentThread().getStackTrace()[3];
      String className = ste.getClassName();
      logger.log(BasicLevel.DEBUG, className.substring(className.lastIndexOf(".")+1) + "." + ste.getMethodName() + " RETURN");
    }
  }
}
